part of katana_module;

/// Module for defining conditions.
///
/// If you specify [T],
/// you can set the type of value that is the source of the condition.
@immutable
abstract class ConditionModule<T> extends Module {
  /// Module for defining conditions.
  const ConditionModule();

  /// Pass [context] and [value] to check the condition.
  ///
  /// If `true` is returned, the process is passed.
  bool check(BuildContext context, T value);
}

/// Module that passes a query and
/// returns `true` if the condition is met.
@immutable
class QueryConditionModule extends ConditionModule<Object?> {
  /// Module that passes a query and
  /// returns `true` if the condition is met.
  const QueryConditionModule(this.query);

  /// Query to use as a condition.
  final Query query;

  @override
  bool check(BuildContext context, Object? value) {
    return query.check(value);
  }

  /// Convert the query condition module information from [DynamicMap].
  @override
  QueryConditionModule? fromMap(DynamicMap map) {
    if (!map.containsKey("query")) {
      return null;
    }
    final query = map.toCollectionQuery();
    if (query == null) {
      return null;
    }

    return QueryConditionModule(query);
  }

  /// Convert the query condition module information to [DynamicMap].
  @override
  DynamicMap toMap() {
    return {
      "query": query.toMap(),
    };
  }
}
